@@ -1,21 +1,13 @@
 # tempfile.py unit tests.
 import tempfile
-import errno
-import io
 import os
-import signal
-import shutil
 import sys
 import re
 import warnings
-import contextlib
 
 import unittest
-from test import test_support as support
+from test import support
 
-warnings.filterwarnings("ignore",
-                        category=RuntimeWarning,
-                        message="mktemp", module=__name__)
 
 if hasattr(os, 'stat'):
     import stat
@@ -28,7 +20,7 @@ has_spawnl = hasattr(os, 'spawnl')
 
 # TEST_FILES may need to be tweaked for systems depending on the maximum
 # number of files that can be opened at one time (see ulimit -n)
-if sys.platform in ('openbsd3', 'openbsd4', 'openbsd5', 'openbsd6', 'openbsd7'):
+if sys.platform in ('openbsd3', 'openbsd4', 'openbsd5'):
     TEST_FILES = 48
 else:
     TEST_FILES = 100
@@ -42,6 +34,16 @@ class TC(unittest.TestCase):
 
     str_check = re.compile(r"[a-zA-Z0-9_-]{6}$")
 
+    def setUp(self):
+        self._warnings_manager = support.check_warnings()
+        self._warnings_manager.__enter__()
+        warnings.filterwarnings("ignore", category=RuntimeWarning,
+                                message="mktemp", module=__name__)
+
+    def tearDown(self):
+        self._warnings_manager.__exit__(None, None, None)
+
+
     def failOnException(self, what, ei=None):
         if ei is None:
             ei = sys.exc_info()
@@ -83,7 +85,8 @@ class test_exports(TC):
             "gettempdir" : 1,
             "tempdir" : 1,
             "template" : 1,
-            "SpooledTemporaryFile" : 1
+            "SpooledTemporaryFile" : 1,
+            "TemporaryDirectory" : 1,
         }
 
         unexp = []
@@ -101,10 +104,11 @@ class test__RandomNameSequence(TC):
 
     def setUp(self):
         self.r = tempfile._RandomNameSequence()
+        super().setUp()
 
     def test_get_six_char_str(self):
         # _RandomNameSequence returns a six-character string
-        s = self.r.next()
+        s = next(self.r)
         self.nameCheck(s, '', '', '')
 
     def test_many(self):
@@ -112,13 +116,13 @@ class test__RandomNameSequence(TC):
 
         dict = {}
         r = self.r
-        for i in xrange(TEST_FILES):
-            s = r.next()
+        for i in range(TEST_FILES):
+            s = next(r)
             self.nameCheck(s, '', '', '')
             self.assertNotIn(s, dict)
             dict[s] = 1
 
-    def test_supports_iter(self):
+    def supports_iter(self):
         # _RandomNameSequence supports the iterator protocol
 
         i = 0
@@ -131,44 +135,6 @@ class test__RandomNameSequence(TC):
         except:
             self.failOnException("iteration")
 
-    @unittest.skipUnless(hasattr(os, 'fork'),
-        "os.fork is required for this test")
-    def test_process_awareness(self):
-        # ensure that the random source differs between
-        # child and parent.
-        read_fd, write_fd = os.pipe()
-        pid = None
-        try:
-            pid = os.fork()
-            if not pid:
-                # child process
-                os.close(read_fd)
-                os.write(write_fd, next(self.r).encode("ascii"))
-                os.close(write_fd)
-                # bypass the normal exit handlers- leave those to
-                # the parent.
-                os._exit(0)
-
-            # parent process
-            parent_value = next(self.r)
-            child_value = os.read(read_fd, len(parent_value)).decode("ascii")
-        finally:
-            if pid:
-                # best effort to ensure the process can't bleed out
-                # via any bugs above
-                try:
-                    os.kill(pid, signal.SIGKILL)
-                except EnvironmentError:
-                    pass
-
-                # Read the process exit status to avoid zombie process
-                os.waitpid(pid, 0)
-
-            os.close(read_fd)
-            os.close(write_fd)
-        self.assertNotEqual(child_value, parent_value)
-
-
 test_classes.append(test__RandomNameSequence)
 
 
@@ -182,7 +148,7 @@ class test__candidate_tempdir_list(TC):
 
         self.assertFalse(len(cand) == 0)
         for c in cand:
-            self.assertIsInstance(c, basestring)
+            self.assertIsInstance(c, str)
 
     def test_wanted_dirs(self):
         # _candidate_tempdir_list contains the expected directories
@@ -213,50 +179,8 @@ class test__candidate_tempdir_list(TC):
 
 test_classes.append(test__candidate_tempdir_list)
 
-# We test _get_default_tempdir some more by testing gettempdir.
 
-class TestGetDefaultTempdir(TC):
-    """Test _get_default_tempdir()."""
-
-    def test_no_files_left_behind(self):
-        # use a private empty directory
-        our_temp_directory = tempfile.mkdtemp()
-        try:
-            # force _get_default_tempdir() to consider our empty directory
-            def our_candidate_list():
-                return [our_temp_directory]
-
-            with support.swap_attr(tempfile, "_candidate_tempdir_list",
-                                   our_candidate_list):
-                # verify our directory is empty after _get_default_tempdir()
-                tempfile._get_default_tempdir()
-                self.assertEqual(os.listdir(our_temp_directory), [])
-
-                def raise_OSError(*args, **kwargs):
-                    raise OSError(-1)
-
-                with support.swap_attr(io, "open", raise_OSError):
-                    # test again with failing io.open()
-                    with self.assertRaises(IOError) as cm:
-                        tempfile._get_default_tempdir()
-                    self.assertEqual(cm.exception.errno, errno.ENOENT)
-                    self.assertEqual(os.listdir(our_temp_directory), [])
-
-                def bad_writer(*args, **kwargs):
-                    fp = orig_open(*args, **kwargs)
-                    fp.write = raise_OSError
-                    return fp
-
-                with support.swap_attr(io, "open", bad_writer) as orig_open:
-                    # test again with failing write()
-                    with self.assertRaises(IOError) as cm:
-                        tempfile._get_default_tempdir()
-                    self.assertEqual(cm.exception.errno, errno.ENOENT)
-                    self.assertEqual(os.listdir(our_temp_directory), [])
-        finally:
-            shutil.rmtree(our_temp_directory)
-
-test_classes.append(TestGetDefaultTempdir)
+# We test _get_default_tempdir by testing gettempdir.
 
 
 class test__get_candidate_names(TC):
@@ -277,58 +201,7 @@ class test__get_candidate_names(TC):
 test_classes.append(test__get_candidate_names)
 
 
-@contextlib.contextmanager
-def _inside_empty_temp_dir():
-    dir = tempfile.mkdtemp()
-    try:
-        with support.swap_attr(tempfile, 'tempdir', dir):
-            yield
-    finally:
-        support.rmtree(dir)
-
-
-def _mock_candidate_names(*names):
-    return support.swap_attr(tempfile,
-                             '_get_candidate_names',
-                             lambda: iter(names))
-
-
-class TestBadTempdir:
-
-    def test_read_only_directory(self):
-        with _inside_empty_temp_dir():
-            oldmode = mode = os.stat(tempfile.tempdir).st_mode
-            mode &= ~(stat.S_IWUSR | stat.S_IWGRP | stat.S_IWOTH)
-            os.chmod(tempfile.tempdir, mode)
-            try:
-                if os.access(tempfile.tempdir, os.W_OK):
-                    self.skipTest("can't set the directory read-only")
-                with self.assertRaises(OSError) as cm:
-                    self.make_temp()
-                self.assertIn(cm.exception.errno, (errno.EPERM, errno.EACCES))
-                self.assertEqual(os.listdir(tempfile.tempdir), [])
-            finally:
-                os.chmod(tempfile.tempdir, oldmode)
-
-    def test_nonexisting_directory(self):
-        with _inside_empty_temp_dir():
-            tempdir = os.path.join(tempfile.tempdir, 'nonexistent')
-            with support.swap_attr(tempfile, 'tempdir', tempdir):
-                with self.assertRaises(OSError) as cm:
-                    self.make_temp()
-                self.assertEqual(cm.exception.errno, errno.ENOENT)
-
-    def test_non_directory(self):
-        with _inside_empty_temp_dir():
-            tempdir = os.path.join(tempfile.tempdir, 'file')
-            open(tempdir, 'wb').close()
-            with support.swap_attr(tempfile, 'tempdir', tempdir):
-                with self.assertRaises(OSError) as cm:
-                    self.make_temp()
-                self.assertIn(cm.exception.errno, (errno.ENOTDIR, errno.ENOENT))
-
-
-class test__mkstemp_inner(TestBadTempdir, TC):
+class test__mkstemp_inner(TC):
     """Test the internal function _mkstemp_inner."""
 
     class mkstemped:
@@ -363,15 +236,15 @@ class test__mkstemp_inner(TestBadTempdir, TC):
 
     def test_basic(self):
         # _mkstemp_inner can create files
-        self.do_create().write("blat")
-        self.do_create(pre="a").write("blat")
-        self.do_create(suf="b").write("blat")
-        self.do_create(pre="a", suf="b").write("blat")
-        self.do_create(pre="aa", suf=".txt").write("blat")
+        self.do_create().write(b"blat")
+        self.do_create(pre="a").write(b"blat")
+        self.do_create(suf="b").write(b"blat")
+        self.do_create(pre="a", suf="b").write(b"blat")
+        self.do_create(pre="aa", suf=".txt").write(b"blat")
 
     def test_basic_many(self):
         # _mkstemp_inner can create many files (stochastic)
-        extant = range(TEST_FILES)
+        extant = list(range(TEST_FILES))
         for i in extant:
             extant[i] = self.do_create(pre="aa")
 
@@ -379,17 +252,18 @@ class test__mkstemp_inner(TestBadTempdir, TC):
         # _mkstemp_inner can create files in a user-selected directory
         dir = tempfile.mkdtemp()
         try:
-            self.do_create(dir=dir).write("blat")
+            self.do_create(dir=dir).write(b"blat")
         finally:
             os.rmdir(dir)
 
-    @unittest.skipUnless(has_stat, 'os.stat not available')
     def test_file_mode(self):
         # _mkstemp_inner creates files with the proper mode
+        if not has_stat:
+            return            # ugh, can't use SkipTest.
 
         file = self.do_create()
         mode = stat.S_IMODE(os.stat(file.name).st_mode)
-        expected = 0600
+        expected = 0o600
         if sys.platform in ('win32', 'os2emx'):
             # There's no distinction among 'user', 'group' and 'world';
             # replicate the 'user' bits.
@@ -397,9 +271,10 @@ class test__mkstemp_inner(TestBadTempdir, TC):
             expected = user * (1 + 8 + 64)
         self.assertEqual(mode, expected)
 
-    @unittest.skipUnless(has_spawnl, 'os.spawnl not available')
     def test_noinherit(self):
         # _mkstemp_inner file handles are not inherited by child processes
+        if not has_spawnl:
+            return            # ugh, can't use SkipTest.
 
         if support.verbose:
             v="v"
@@ -434,43 +309,17 @@ class test__mkstemp_inner(TestBadTempdir, TC):
                     "child process caught fatal signal %d" % -retval)
         self.assertFalse(retval > 0, "child process reports failure %d"%retval)
 
-    @unittest.skipUnless(has_textmode, "text mode not available")
     def test_textmode(self):
         # _mkstemp_inner can create files in text mode
+        if not has_textmode:
+            return            # ugh, can't use SkipTest.
 
-        self.do_create(bin=0).write("blat\n")
-        # XXX should test that the file really is a text file
-
-    def make_temp(self):
-        return tempfile._mkstemp_inner(tempfile.gettempdir(),
-                                       tempfile.template,
-                                       '',
-                                       tempfile._bin_openflags)
-
-    def test_collision_with_existing_file(self):
-        # _mkstemp_inner tries another name when a file with
-        # the chosen name already exists
-        with _inside_empty_temp_dir(), \
-             _mock_candidate_names('aaa', 'aaa', 'bbb'):
-            (fd1, name1) = self.make_temp()
-            os.close(fd1)
-            self.assertTrue(name1.endswith('aaa'))
-
-            (fd2, name2) = self.make_temp()
-            os.close(fd2)
-            self.assertTrue(name2.endswith('bbb'))
-
-    def test_collision_with_existing_directory(self):
-        # _mkstemp_inner tries another name when a directory with
-        # the chosen name already exists
-        with _inside_empty_temp_dir(), \
-             _mock_candidate_names('aaa', 'aaa', 'bbb'):
-            dir = tempfile.mkdtemp()
-            self.assertTrue(dir.endswith('aaa'))
-
-            (fd, name) = self.make_temp()
-            os.close(fd)
-            self.assertTrue(name.endswith('bbb'))
+        # A text file is truncated at the first Ctrl+Z byte
+        f = self.do_create(bin=0)
+        f.write(b"blat\x1a")
+        f.write(b"extra\n")
+        os.lseek(f.fd, 0, os.SEEK_SET)
+        self.assertEqual(os.read(f.fd, 20), b"blat")
 
 test_classes.append(test__mkstemp_inner)
 
@@ -482,7 +331,7 @@ class test_gettempprefix(TC):
         # gettempprefix returns a nonempty prefix string
         p = tempfile.gettempprefix()
 
-        self.assertIsInstance(p, basestring)
+        self.assertIsInstance(p, str)
         self.assertTrue(len(p) > 0)
 
     def test_usable_template(self):
@@ -527,7 +376,7 @@ class test_gettempdir(TC):
         # gettempdir.
         try:
             file = tempfile.NamedTemporaryFile()
-            file.write("blat")
+            file.write(b"blat")
             file.close()
         except:
             self.failOnException("create file in %s" % tempfile.gettempdir())
@@ -583,12 +432,9 @@ class test_mkstemp(TC):
 test_classes.append(test_mkstemp)
 
 
-class test_mkdtemp(TestBadTempdir, TC):
+class test_mkdtemp(TC):
     """Test mkdtemp()."""
 
-    def make_temp(self):
-        return tempfile.mkdtemp()
-
     def do_create(self, dir=None, pre="", suf=""):
         if dir is None:
             dir = tempfile.gettempdir()
@@ -614,13 +460,13 @@ class test_mkdtemp(TestBadTempdir, TC):
 
     def test_basic_many(self):
         # mkdtemp can create many directories (stochastic)
-        extant = range(TEST_FILES)
+        extant = list(range(TEST_FILES))
         try:
             for i in extant:
                 extant[i] = self.do_create(pre="aa")
         finally:
             for i in extant:
-                if(isinstance(i, basestring)):
+                if(isinstance(i, str)):
                     os.rmdir(i)
 
     def test_choose_directory(self):
@@ -631,15 +477,16 @@ class test_mkdtemp(TestBadTempdir, TC):
         finally:
             os.rmdir(dir)
 
-    @unittest.skipUnless(has_stat, 'os.stat not available')
     def test_mode(self):
         # mkdtemp creates directories with the proper mode
+        if not has_stat:
+            return            # ugh, can't use SkipTest.
 
         dir = self.do_create()
         try:
             mode = stat.S_IMODE(os.stat(dir).st_mode)
-            mode &= 0777 # Mask off sticky bits inherited from /tmp
-            expected = 0700
+            mode &= 0o777 # Mask off sticky bits inherited from /tmp
+            expected = 0o700
             if sys.platform in ('win32', 'os2emx'):
                 # There's no distinction among 'user', 'group' and 'world';
                 # replicate the 'user' bits.
@@ -649,27 +496,6 @@ class test_mkdtemp(TestBadTempdir, TC):
         finally:
             os.rmdir(dir)
 
-    def test_collision_with_existing_file(self):
-        # mkdtemp tries another name when a file with
-        # the chosen name already exists
-        with _inside_empty_temp_dir(), \
-             _mock_candidate_names('aaa', 'aaa', 'bbb'):
-            file = tempfile.NamedTemporaryFile(delete=False)
-            file.close()
-            self.assertTrue(file.name.endswith('aaa'))
-            dir = tempfile.mkdtemp()
-            self.assertTrue(dir.endswith('bbb'))
-
-    def test_collision_with_existing_directory(self):
-        # mkdtemp tries another name when a directory with
-        # the chosen name already exists
-        with _inside_empty_temp_dir(), \
-             _mock_candidate_names('aaa', 'aaa', 'bbb'):
-            dir1 = tempfile.mkdtemp()
-            self.assertTrue(dir1.endswith('aaa'))
-            dir2 = tempfile.mkdtemp()
-            self.assertTrue(dir2.endswith('bbb'))
-
 test_classes.append(test_mkdtemp)
 
 
@@ -680,11 +506,13 @@ class test_mktemp(TC):
     # We must also suppress the RuntimeWarning it generates.
     def setUp(self):
         self.dir = tempfile.mkdtemp()
+        super().setUp()
 
     def tearDown(self):
         if self.dir:
             os.rmdir(self.dir)
             self.dir = None
+        super().tearDown()
 
     class mktemped:
         _unlink = os.unlink
@@ -694,7 +522,7 @@ class test_mktemp(TC):
             self.name = tempfile.mktemp(dir=dir, prefix=pre, suffix=suf)
             # Create the file.  This will raise an exception if it's
             # mysteriously appeared in the meanwhile.
-            os.close(os.open(self.name, self._bflags, 0600))
+            os.close(os.open(self.name, self._bflags, 0o600))
 
         def __del__(self):
             self._unlink(self.name)
@@ -718,7 +546,7 @@ class test_mktemp(TC):
 
     def test_many(self):
         # mktemp can choose many usable file names (stochastic)
-        extant = range(TEST_FILES)
+        extant = list(range(TEST_FILES))
         for i in extant:
             extant[i] = self.do_create(pre="aa")
 
@@ -771,7 +599,7 @@ class test_NamedTemporaryFile(TC):
         dir = tempfile.mkdtemp()
         try:
             f = tempfile.NamedTemporaryFile(dir=dir)
-            f.write('blat')
+            f.write(b'blat')
             f.close()
             self.assertFalse(os.path.exists(f.name),
                         "NamedTemporaryFile %s exists after close" % f.name)
@@ -785,7 +613,7 @@ class test_NamedTemporaryFile(TC):
         try:
             f = tempfile.NamedTemporaryFile(dir=dir, delete=False)
             tmp = f.name
-            f.write('blat')
+            f.write(b'blat')
             f.close()
             self.assertTrue(os.path.exists(f.name),
                         "NamedTemporaryFile %s missing after close" % f.name)
@@ -797,7 +625,7 @@ class test_NamedTemporaryFile(TC):
     def test_multiple_close(self):
         # A NamedTemporaryFile can be closed many times without error
         f = tempfile.NamedTemporaryFile()
-        f.write('abc\n')
+        f.write(b'abc\n')
         f.close()
         try:
             f.close()
@@ -815,32 +643,6 @@ class test_NamedTemporaryFile(TC):
                 pass
         self.assertRaises(ValueError, use_closed)
 
-    def test_no_leak_fd(self):
-        # Issue #21058: don't leak file descriptor when fdopen() fails
-        old_close = os.close
-        old_fdopen = os.fdopen
-        closed = []
-        def close(fd):
-            old_close(fd)
-            closed.append(fd)
-        def fdopen(*args):
-            raise ValueError()
-        os.close = close
-        os.fdopen = fdopen
-        try:
-            self.assertRaises(ValueError, tempfile.NamedTemporaryFile)
-            self.assertEqual(len(closed), 1)
-        finally:
-            os.close = old_close
-            os.fdopen = old_fdopen
-
-    def test_bad_mode(self):
-        dir = tempfile.mkdtemp()
-        self.addCleanup(support.rmtree, dir)
-        with self.assertRaises(TypeError):
-            tempfile.NamedTemporaryFile(mode=(), dir=dir)
-        self.assertEqual(os.listdir(dir), [])
-
     # How to test the mode and bufsize parameters?
 
 test_classes.append(test_NamedTemporaryFile)
@@ -872,11 +674,11 @@ class test_SpooledTemporaryFile(TC):
         try:
             f = tempfile.SpooledTemporaryFile(max_size=10, dir=dir)
             self.assertFalse(f._rolled)
-            f.write('blat ' * 5)
+            f.write(b'blat ' * 5)
             self.assertTrue(f._rolled)
             filename = f.name
             f.close()
-            self.assertFalse(os.path.exists(filename),
+            self.assertFalse(isinstance(filename, str) and os.path.exists(filename),
                         "SpooledTemporaryFile %s exists after close" % filename)
         finally:
             os.rmdir(dir)
@@ -887,7 +689,7 @@ class test_SpooledTemporaryFile(TC):
         self.assertFalse(f._rolled)
         for i in range(5):
             f.seek(0, 0)
-            f.write('x' * 20)
+            f.write(b'x' * 20)
         self.assertFalse(f._rolled)
 
     def test_write_sequential(self):
@@ -895,11 +697,11 @@ class test_SpooledTemporaryFile(TC):
         # over afterward
         f = self.do_create(max_size=30)
         self.assertFalse(f._rolled)
-        f.write('x' * 20)
+        f.write(b'x' * 20)
         self.assertFalse(f._rolled)
-        f.write('x' * 10)
+        f.write(b'x' * 10)
         self.assertFalse(f._rolled)
-        f.write('x')
+        f.write(b'x')
         self.assertTrue(f._rolled)
 
     def test_writelines(self):
@@ -919,17 +721,6 @@ class test_SpooledTemporaryFile(TC):
         f.write(b'x')
         self.assertTrue(f._rolled)
 
-    def test_xreadlines(self):
-        f = self.do_create(max_size=20)
-        f.write(b'abc\n' * 5)
-        f.seek(0)
-        self.assertFalse(f._rolled)
-        self.assertEqual(list(f.xreadlines()), [b'abc\n'] * 5)
-        f.write(b'x\ny')
-        self.assertTrue(f._rolled)
-        f.seek(0)
-        self.assertEqual(list(f.xreadlines()), [b'abc\n'] * 5 + [b'x\n', b'y'])
-
     def test_sparse(self):
         # A SpooledTemporaryFile that is written late in the file will extend
         # when that occurs
@@ -937,7 +728,7 @@ class test_SpooledTemporaryFile(TC):
         self.assertFalse(f._rolled)
         f.seek(100, 0)
         self.assertFalse(f._rolled)
-        f.write('x')
+        f.write(b'x')
         self.assertTrue(f._rolled)
 
     def test_fileno(self):
@@ -950,7 +741,7 @@ class test_SpooledTemporaryFile(TC):
     def test_multiple_close_before_rollover(self):
         # A SpooledTemporaryFile can be closed many times without error
         f = tempfile.SpooledTemporaryFile()
-        f.write('abc\n')
+        f.write(b'abc\n')
         self.assertFalse(f._rolled)
         f.close()
         try:
@@ -962,7 +753,7 @@ class test_SpooledTemporaryFile(TC):
     def test_multiple_close_after_rollover(self):
         # A SpooledTemporaryFile can be closed many times without error
         f = tempfile.SpooledTemporaryFile(max_size=1)
-        f.write('abc\n')
+        f.write(b'abc\n')
         self.assertTrue(f._rolled)
         f.close()
         try:
@@ -980,30 +771,41 @@ class test_SpooledTemporaryFile(TC):
         write = f.write
         seek = f.seek
 
-        write("a" * 35)
-        write("b" * 35)
+        write(b"a" * 35)
+        write(b"b" * 35)
         seek(0, 0)
-        self.assertTrue(read(70) == 'a'*35 + 'b'*35)
+        self.assertEqual(read(70), b'a'*35 + b'b'*35)
 
-    def test_properties(self):
-        f = tempfile.SpooledTemporaryFile(max_size=10)
-        f.write(b'x' * 10)
+    def test_text_mode(self):
+        # Creating a SpooledTemporaryFile with a text mode should produce
+        # a file object reading and writing (Unicode) text strings.
+        f = tempfile.SpooledTemporaryFile(mode='w+', max_size=10)
+        f.write("abc\n")
+        f.seek(0)
+        self.assertEqual(f.read(), "abc\n")
+        f.write("def\n")
+        f.seek(0)
+        self.assertEqual(f.read(), "abc\ndef\n")
+        f.write("xyzzy\n")
+        f.seek(0)
+        self.assertEqual(f.read(), "abc\ndef\nxyzzy\n")
+        # Check that Ctrl+Z doesn't truncate the file
+        f.write("foo\x1abar\n")
+        f.seek(0)
+        self.assertEqual(f.read(), "abc\ndef\nxyzzy\nfoo\x1abar\n")
+
+    def test_text_newline_and_encoding(self):
+        f = tempfile.SpooledTemporaryFile(mode='w+', max_size=10,
+                                          newline='', encoding='utf-8')
+        f.write("\u039B\r\n")
+        f.seek(0)
+        self.assertEqual(f.read(), "\u039B\r\n")
         self.assertFalse(f._rolled)
-        self.assertEqual(f.mode, 'w+b')
-        self.assertIsNone(f.name)
-        with self.assertRaises(AttributeError):
-            f.newlines
-        with self.assertRaises(AttributeError):
-            f.encoding
 
-        f.write(b'x')
+        f.write("\u039B" * 20 + "\r\n")
+        f.seek(0)
+        self.assertEqual(f.read(), "\u039B\r\n" + ("\u039B" * 20) + "\r\n")
         self.assertTrue(f._rolled)
-        self.assertEqual(f.mode, 'w+b')
-        self.assertIsNotNone(f.name)
-        with self.assertRaises(AttributeError):
-            f.newlines
-        with self.assertRaises(AttributeError):
-            f.encoding
 
     def test_context_manager_before_rollover(self):
         # A SpooledTemporaryFile can be used as a context manager
@@ -1020,7 +822,7 @@ class test_SpooledTemporaryFile(TC):
         # A SpooledTemporaryFile can be used as a context manager
         with tempfile.SpooledTemporaryFile(max_size=1) as f:
             self.assertFalse(f._rolled)
-            f.write('abc\n')
+            f.write(b'abc\n')
             f.flush()
             self.assertTrue(f._rolled)
             self.assertFalse(f.closed)
@@ -1033,7 +835,7 @@ class test_SpooledTemporaryFile(TC):
     def test_context_manager_after_rollover(self):
         # A SpooledTemporaryFile can be used as a context manager
         f = tempfile.SpooledTemporaryFile(max_size=1)
-        f.write('abc\n')
+        f.write(b'abc\n')
         f.flush()
         self.assertTrue(f._rolled)
         with f:
@@ -1063,7 +865,7 @@ class test_TemporaryFile(TC):
         # TemporaryFile creates files with no names (on this system)
         dir = tempfile.mkdtemp()
         f = tempfile.TemporaryFile(dir=dir)
-        f.write('blat')
+        f.write(b'blat')
 
         # Sneaky: because this file has no name, it should not prevent
         # us from removing the directory it was created in.
@@ -1079,7 +881,7 @@ class test_TemporaryFile(TC):
     def test_multiple_close(self):
         # A TemporaryFile can be closed many times without error
         f = tempfile.TemporaryFile()
-        f.write('abc\n')
+        f.write(b'abc\n')
         f.close()
         try:
             f.close()
@@ -1088,11 +890,190 @@ class test_TemporaryFile(TC):
             self.failOnException("close")
 
     # How to test the mode and bufsize parameters?
+    def test_mode_and_encoding(self):
+
+        def roundtrip(input, *args, **kwargs):
+            with tempfile.TemporaryFile(*args, **kwargs) as fileobj:
+                fileobj.write(input)
+                fileobj.seek(0)
+                self.assertEqual(input, fileobj.read())
+
+        roundtrip(b"1234", "w+b")
+        roundtrip("abdc\n", "w+")
+        roundtrip("\u039B", "w+", encoding="utf-16")
+        roundtrip("foo\r\n", "w+", newline="")
 
 
 if tempfile.NamedTemporaryFile is not tempfile.TemporaryFile:
     test_classes.append(test_TemporaryFile)
 
+
+# Helper for test_del_on_shutdown
+class NulledModules:
+    def __init__(self, *modules):
+        self.refs = [mod.__dict__ for mod in modules]
+        self.contents = [ref.copy() for ref in self.refs]
+
+    def __enter__(self):
+        for d in self.refs:
+            for key in d:
+                d[key] = None
+
+    def __exit__(self, *exc_info):
+        for d, c in zip(self.refs, self.contents):
+            d.clear()
+            d.update(c)
+
+class test_TemporaryDirectory(TC):
+    """Test TemporaryDirectory()."""
+
+    def do_create(self, dir=None, pre="", suf="", recurse=1):
+        if dir is None:
+            dir = tempfile.gettempdir()
+        try:
+            tmp = tempfile.TemporaryDirectory(dir=dir, prefix=pre, suffix=suf)
+        except:
+            self.failOnException("TemporaryDirectory")
+        self.nameCheck(tmp.name, dir, pre, suf)
+        # Create a subdirectory and some files
+        if recurse:
+            self.do_create(tmp.name, pre, suf, recurse-1)
+        with open(os.path.join(tmp.name, "test.txt"), "wb") as f:
+            f.write(b"Hello world!")
+        return tmp
+
+    def test_mkdtemp_failure(self):
+        # Check no additional exception if mkdtemp fails
+        # Previously would raise AttributeError instead
+        # (noted as part of Issue #10188)
+        with tempfile.TemporaryDirectory() as nonexistent:
+            pass
+        with self.assertRaises(os.error):
+            tempfile.TemporaryDirectory(dir=nonexistent)
+
+    def test_explicit_cleanup(self):
+        # A TemporaryDirectory is deleted when cleaned up
+        dir = tempfile.mkdtemp()
+        try:
+            d = self.do_create(dir=dir)
+            self.assertTrue(os.path.exists(d.name),
+                            "TemporaryDirectory %s does not exist" % d.name)
+            d.cleanup()
+            self.assertFalse(os.path.exists(d.name),
+                        "TemporaryDirectory %s exists after cleanup" % d.name)
+        finally:
+            os.rmdir(dir)
+
+    @support.skip_unless_symlink
+    def test_cleanup_with_symlink_to_a_directory(self):
+        # cleanup() should not follow symlinks to directories (issue #12464)
+        d1 = self.do_create()
+        d2 = self.do_create()
+
+        # Symlink d1/foo -> d2
+        os.symlink(d2.name, os.path.join(d1.name, "foo"))
+
+        # This call to cleanup() should not follow the "foo" symlink
+        d1.cleanup()
+
+        self.assertFalse(os.path.exists(d1.name),
+                         "TemporaryDirectory %s exists after cleanup" % d1.name)
+        self.assertTrue(os.path.exists(d2.name),
+                        "Directory pointed to by a symlink was deleted")
+        self.assertEqual(os.listdir(d2.name), ['test.txt'],
+                         "Contents of the directory pointed to by a symlink "
+                         "were deleted")
+        d2.cleanup()
+
+    @support.cpython_only
+    def test_del_on_collection(self):
+        # A TemporaryDirectory is deleted when garbage collected
+        dir = tempfile.mkdtemp()
+        try:
+            d = self.do_create(dir=dir)
+            name = d.name
+            del d # Rely on refcounting to invoke __del__
+            self.assertFalse(os.path.exists(name),
+                        "TemporaryDirectory %s exists after __del__" % name)
+        finally:
+            os.rmdir(dir)
+
+    @unittest.expectedFailure # See issue #10188
+    def test_del_on_shutdown(self):
+        # A TemporaryDirectory may be cleaned up during shutdown
+        # Make sure it works with the relevant modules nulled out
+        with self.do_create() as dir:
+            d = self.do_create(dir=dir)
+            # Mimic the nulling out of modules that
+            # occurs during system shutdown
+            modules = [os, os.path]
+            if has_stat:
+                modules.append(stat)
+            # Currently broken, so suppress the warning
+            # that is otherwise emitted on stdout
+            with support.captured_stderr() as err:
+                with NulledModules(*modules):
+                    d.cleanup()
+            # Currently broken, so stop spurious exception by
+            # indicating the object has already been closed
+            d._closed = True
+            # And this assert will fail, as expected by the
+            # unittest decorator...
+            self.assertFalse(os.path.exists(d.name),
+                        "TemporaryDirectory %s exists after cleanup" % d.name)
+
+    def test_warnings_on_cleanup(self):
+        # Two kinds of warning on shutdown
+        #   Issue 10888: may write to stderr if modules are nulled out
+        #   ResourceWarning will be triggered by __del__
+        with self.do_create() as dir:
+            if os.sep != '\\':
+                # Embed a backslash in order to make sure string escaping
+                # in the displayed error message is dealt with correctly
+                suffix = '\\check_backslash_handling'
+            else:
+                suffix = ''
+            d = self.do_create(dir=dir, suf=suffix)
+
+            #Check for the Issue 10888 message
+            modules = [os, os.path]
+            if has_stat:
+                modules.append(stat)
+            with support.captured_stderr() as err:
+                with NulledModules(*modules):
+                    d.cleanup()
+            message = err.getvalue().replace('\\\\', '\\')
+            self.assertIn("while cleaning up",  message)
+            self.assertIn(d.name,  message)
+
+            # Check for the resource warning
+            with support.check_warnings(('Implicitly', ResourceWarning), quiet=False):
+                warnings.filterwarnings("always", category=ResourceWarning)
+                d.__del__()
+            self.assertFalse(os.path.exists(d.name),
+                        "TemporaryDirectory %s exists after __del__" % d.name)
+
+    def test_multiple_close(self):
+        # Can be cleaned-up many times without error
+        d = self.do_create()
+        d.cleanup()
+        try:
+            d.cleanup()
+            d.cleanup()
+        except:
+            self.failOnException("cleanup")
+
+    def test_context_manager(self):
+        # Can be used as a context manager
+        d = self.do_create()
+        with d as name:
+            self.assertTrue(os.path.exists(name))
+            self.assertEqual(name, d.name)
+        self.assertFalse(os.path.exists(name))
+
+
+test_classes.append(test_TemporaryDirectory)
+
 def test_main():
     support.run_unittest(*test_classes)
 
